;*************************************************************************
;*
;* Purebasic Eventhandler
;*
;*************************************************************************
;*
;* Programname       : PureBasic Eventhandler
;* Filename          : Eventhandler.pbi
;* Filetype          : Includefile   [Main source, Includefile, Data file]
;* Version           : 1.05
;* Date              : 22.07.2012
;* Autor             : Markus Haacke (www.gismeth.de)
;* Programming lang. : Purebasic 4.50+
;* Compile as        : -
;* -----------------------------------------------------------------------
;* BESCHREIBUNG:
;*
;* siehe unten
;*
;* -----------------------------------------------------------------------
;* Historie:
;* 1.05 - chg 22.07.2012:
;*      - chg Die beiden Windows-spezifischen Funktionen EventwParam()und EventlParam() werden nur noch unter Windows mitkompiliert.
;*            Die zugehrigen Strukturfelder \iEventwParam.i und \iEventlParam.i enthalten unter non-Windows Installationen den Wert 0
;* 1.04 - chg 06.07.2010:
;*  					Alle evh_SetXYZEvent() Befehle enthalten nun den Parameter iWindowNr
;*      - add Neuen Befehl evh_RemoveEvent() zugefgt
;*      - chg evh_Init() mu nicht mehr manuell aufgerufen werden
;* 1.03 - chg 03.07.2010:
;*  					evh_EventArgsStruct erweitert um iEventTimer
;* 1.02 - chg 28.05.2010:
;*  					evh_ProcessEvents() hat einen neuen Parameter bekommen: *stEvents
;* 1.01 - chg 26.05.2010:
;*  					Die einzelnen Parameter der Eventprozeduren auf evh_EventArgsStruct umgestellt
;* 1.00 - rel 23.05.2010:
;*  					Erste Version
;*
;*************************************************************************

;*************************************************************************
;* Informationen zur Benutzung
;*************************************************************************
; Mit diesem Eventhandler wird die Auswertung von Window-, Button- und Menevents erleichtert.
;
; Nachdem die GUI des eigenen Programms erzeugt wurde, knnen den Gadgets, Fenstern und
; Meneintrgen mit den folgenden Befehlen eigene Eventprozeduren zugeordnet werden.
; 
;   evh_SetWindowEvent(iWindowNr, iEventType, *EventProcedure)
;   evh_SetMenuEvent  (iWindowNr, iMenuitemNr, *EventProcedure)
;   evh_SetGadgetEvent(iWindowNr, iGadgetNr, iEventType, *EventProcedure)
; 
; iWindowNr, iGadgetNr bzw. iMenuitemNr stellen dabei die Nummern des jeweiligen Fensters, Gadgets oder 
; Meneintrags dar.
;
; iEventType beschreibt die PB Eventkonstante auf die reagiert werden soll (z.B. #PB_EventType_LeftClick)
; Folgende Eventtypen knnen bei den jeweiligen Befehlen benutzt werden:
;
;   evh_SetWindowEvent(): Alle Events, die der Purebasic-Befehl WindowEvent() zurck gibt
;   evh_SetMenuEvent()  : Hier ist kein Eventtyp anzugeben, es wird nur die Menuitem-Nummer ausgewertet
;   evh_SetGadgetEvent(): Alle Events, die der Purebasic-Befehl EventType() zurck gibt
;
; In *EventProcedure gibt man die Adresse der Prozedur an, die bei Auftreten des Events aufgerufen werden soll
; (sie ist anzugeben mittels @Prozedurname()).
;
; Jeder Eventprozedur wird als einziger Parameter ein Zeiger auf die folgende Event-Parameterstruktur bergeben:
;
;   \iEvent
;   \iEventType
;   \iEventWindow
;   \iEventGadget
;   \iEventMenu
;   \iEventTimer
;   \iEventwParam ; enthlt nur unter Windows sinnvolle Werte
;   \iEventlParam ; enthlt nur unter Windows sinnvolle Werte
;
; (Der Strukturtyp lautet "evh_EventArgsStruct" und ist im Eventhandler-Include definiert)
;
; Eine Eventprozedur ist somit immer nach folgendem Schema aufzubauen (Beispiel):
;
;   Procedure Name(*stEventArgs.evh_EventArgsStruct)
;   ...
;   EndProcedure
;
; Die Felder der Struktur entsprechen den quivalenten PB-Befehlen WaitWindowEvent(), EventType(),
; EventWindow(), usw. und enthalten genau das jeweilige Ergebnis dieser PB-Befehle.
; Die Felder iEventwParam und iEventlParam enthalten nur gltige Werte, wenn das Include auf einem Windwos PC
; kompiliert wird. Die zugehrigen PB Befehle EventwParam() und EventlParam() sind undokumentierte Befehle und
; funktionieren nur unter Windows. Auf anderen Betriebssystemen werden diese Felder mit 0 initialisiert.
;
; Die Felder \iEventGadget bzw. \iEventMenu enthalten nur sinnvolle Werte, wenn das Feld \iEvent entweder
; #PB_Event_Gadget oder #PB_Event_Menu enthlt.
;
; WICHTIG:
; Bitte nicht vergessen den Parameter (*stEventArgs.evh_EventArgsStruct) immer in die eigenen Eventprozeduren
; einzubauen, da es sonst zwangslufig zu einem Rcksprungfehler kommt! (=> Das Programm strzt ab, weil die
; Daten vom Stack nicht abgeholt wurden)
; 
; Um einen registrierten Event wieder aufzuheben, ruft man den Befehl evh_RemoveEvent() auf
;
;   evh_RemoveEvent(iWindowNr, [iItemNr], [iEventType])
;
; Die beiden Parameter iItemNr und iEventType entsprechen den beiden Parametern des jeweiligen 
; evh_SetXYZEvent-Befehls. Also z.B. der GadgetNummer und dem Eventtyp bei evh_SetGadgetEvent()
; 
; Werden die beiden o.g. Parameter weggelassen und nur iWindowNr angegeben, dann werden automatisch
; ALLE Events unregistriert, die zu dem angegebenen Fenster iWindowNr gehren.
; 
; Dies ist ntzlich, wenn man beim Schlieen eines Fensters alle zugehrigen, registrierten Events auf einen
; Schlag unregistrieren mchte.
;
; Der eigentliche Eventloop des Programms beschrnkt sich dann nur noch auf ein zweizeiliges Konstrukt.
; Beispiel:
;
;   While evh_ProcessEvents(25, MainWindow, @OwnEventproc(), 0)
;   Wend
; 
; Mit dem Befehl evh_ProcessEvents(iTimeOut, iObserveWindow, [*OwnEventHandler], [*stEvents]) nimmt der
; Eventhandler seine Arbeit auf. Der Befehl mu innerhalb eines Loops immer wiederkehrend aufgerufen
; werden, genau so wie der PB Befehl WaitWindowEvent().
;
; Die Parameter im Einzelnen: 
;
; iTimeOut: Der Parameter iTimeOut gibt den Timeoutwert in Millisekunden an, die der Eventhandler auf
; eine Nachricht warten soll. Kommt in dieser Zeit keine Nachricht an, dann kehrt das Programm in den
; Loop zurck. Der Eventhandler benutzt den WaitWindowEvent() Befehl, insofern gilt analog das, was in
; der PB Hilfe zu diesem Befehl erklrt wird.
; (Anm.: Wird der Parameter *stEvents benutzt, dann ist der Wert iTimeOut nicht relevant)
; 
; iObserveWindow: Mit iObserveWindow gibt man das Handle des Hauptfensters seiner Applikation an.
; Wird in diesem Fenster ein #PB_Event_CloseWindow ausgelst (= Fenster wird geschlossen), dann gibt der
; Befehl evh_ProcessEvents() ein #False zurck. In allen anderen Fllen ein #True.
; Dadruch wird der obige While/Wend Eventloop ermglicht, der mittels Rckgabewert #False beendet wird.
; 
; *OwnEventHandler (optional): Da man vielleicht nicht alle Events ber den Eventhandler bearbeiten lassen
; mchte, kann man als dritten Parameter die Adresse einer eigenen Eventprozedur angeben, die *nach*
; Abarbeitung der registrierten Events zustzlich aufgerufen wird. Mchte man keine eigene Event Bearbeitungs-
; prozedur benutzen, dann setzt man als dritten Parameter 0 ein.
; 
; Diese eigene Event-Bearbeitungsprozedur bekommt als Parameter ebenfalls einen Zeiger auf eine Event-
; Parameterstruktur vom Typ evh_EventArgsStruct bergeben:
;
;   \iEvent
;   \iEventType
;   \iEventWindow
;   \iEventGadget
;   \iEventMenu
;   \iEventTimer
;   \iEventwParam ; enthlt nur unter Windows sinnvolle Werte
;   \iEventlParam ; enthlt nur unter Windows sinnvolle Werte
; 
; Auch der Inhalt dieser Felder entspricht den quivalenten PB-Befehlen WaitWindowEvent(), EventType(),
; EventWindow(), usw.
; 
; Der Rumpf der eigenen Event-Bearbeitungsprozedur ist somit genau so aufgebaut, wie eine Eventprozedur:
;
;   Procedure Name(*stEventArgs.evh_EventArgsStruct)
;   ...
;   EndProcedure
;
; WICHTIG:
; Bitte nicht vergessen den Parameter (*stEventArgs.evh_EventArgsStruct) immer in die eigenen Event-Bearbeitungs-
; prozedur einzubauen, da es sonst zwangslufig zu einem Rcksprungfehler kommt! (=> Das Programm strzt ab, 
; weil die Daten vom Stack nicht abgeholt wurden)
;
; *stEvents (optional): *stEvents stellt einen Zeiger auf eine Event-Parameterstruktur dar.
; Ist dieser Zeiger =! 0 (zeigt er also auf eine initialisierte Struktur), dann ermittelt die
; Prozedur evh_ProcessEvents() die PureBasic Fensterevents nicht selbst, sondern bearbeitet die Events,
; die mittels der *stEvents-Struktur bergeben wurden.
;
; Hiermit ist es mglich, seine eigene Eventbearbeitung bzw. sogar eigene Events zu programmieren,
; die dann ber den Eventhandler automatisch ausgewertet werden (sofern diese Events zuvor registriert
; wurden.
;
; Die Struktur, auf die *stEvents zeigt ist ebenfalls vom Typ evh_EventArgsStruct und hat folgenden Aufbau:
;
;   \iEvent
;   \iEventType
;   \iEventWindow
;   \iEventGadget
;   \iEventMenu
;   \iEventTimer
;   \iEventwParam ; enthlt nur unter Windows sinnvolle Werte
;   \iEventlParam ; enthlt nur unter Windows sinnvolle Werte
;
; Werden die Parameter *stEvents und *OwnEventHandler nicht genutzt, dann knnen sie weggelassen werden.

;*************************************************************************
;* Prototypes, Constants, Structures  & Variables
;*************************************************************************

; Konstanten zur Verwendung im Event-Verwaltungsarray
Enumeration 
	#EVH_Controltype_Gadget
	#EVH_Controltype_Menu
	#EVH_Controltype_Window
EndEnumeration

; Funktionsvariablen fr den Aufruf der Eventprozeduren
Prototype.i evh_CallEventProcedure(*stEventArgs)
Prototype.i evh_CallOwnEventHandler(*stEventArgs)

; Struktur fr das Event-Verwaltungsarray
Structure evh_Eventstruct
	iWindowNr.i																										; Nummer des Windows zu dem das Control gehrt
	iControlType.i																								; #EVH_Controltype_Gadget, #EVH_Controltype_Menu, #EVH_Controltype_Window
	iControlNr.i																									; Nummer des Gadgets/Menitems/Windows
	iEventType.i																									; Z.B. #PB_EventType_LeftClick, #PB_EventType_RightClick, #PB_Event_SizeWindow usw...
	iEventProcedure.i																							; Adresse der aufzurufenden Prozedur
EndStructure

; Struktur fr die interne Verwaltung
Structure evh_Settingsstruct
	iHandlerInitialized.i																					; Zeigt an, ob der Eventhandler korrekt initialisiert wurde
	iCurrentEntry.i																								; Indexzeiger auf aktuellen Eintrag im Array (= immer der letzte gltige Eintrag)
EndStructure

; Struktur fr die Parameterbergabe an die Eventprozeduren
Structure evh_EventArgsStruct																		; Parameter-Struktur die den Eventprozeduren mittels Pointer bergeben wird.
	iEvent.i
	iEventType.i
	iEventWindow.i
	iEventGadget.i
	iEventMenu.i
	iEventTimer.i
	iEventwParam.i
	iEventlParam.i
EndStructure

Global evh_stSettings.evh_Settingsstruct

;*************************************************************************
;* Procedures
;*************************************************************************
Procedure 	evh_Init()
	; +-----------------------------------------------------------------
	; |Description  : Initialisiert die Variablen des Eventhandlers, falls noch nicht geschehen
	; |             : Die Prozedur wird nur intern aufgerufen!
	; |Arguments    : -
	; |Results      : -
	; +-----------------------------------------------------------------
	
	If Not evh_stSettings\iHandlerInitialized
		Global Dim evh_stEventArray.evh_Eventstruct(0)
		evh_stSettings\iHandlerInitialized = #True
	EndIf

EndProcedure
Procedure.i evh_SetWindowEvent(iWindowNr.i, iEventType.i, *EventProcedure)
	; +-----------------------------------------------------------------
	; |Description  : Ordnet einem Fenster eine Eventprozedur zu
	; |Arguments    : iWindowNr      : Nummer des Fensters
	; |             : iEventType     : Eventtyp-Konstante auf die reagiert werden soll
	; |             : *EventProcedure: Zeiger auf die Prozedur, die bei diesem Eventtyp aufgerufen werden soll
	; |Results      : #False, wenn evh_Init() zuvor nicht aufgerufen wurde, sonst #True
	; +-----------------------------------------------------------------
	Protected iCount.i

	evh_Init()
	
	; Wenn der Event schon registriert ist, dann die neue Eventprozedur zuordnen...
	For iCount = 0 To evh_stSettings\iCurrentEntry - 1
		If evh_stEventArray(iCount)\iControlNr = iWindowNr And evh_stEventArray(iCount)\iEventType = iEventType
			evh_stEventArray(iCount)\iEventProcedure = *EventProcedure
			ProcedureReturn #True
		EndIf
	Next iCount

	; ... ansonsten den Event registrieren
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iWindowNr = iWindowNr
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iControlType = #EVH_Controltype_Window
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iControlNr = iWindowNr
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iEventType = iEventType
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iEventProcedure = *EventProcedure
	evh_stSettings\iCurrentEntry + 1
	ReDim evh_stEventArray.evh_Eventstruct(evh_stSettings\iCurrentEntry)

	ProcedureReturn #True
EndProcedure
Procedure.i evh_SetGadgetEvent(iWindowNr.i, iGadgetNr.i, iEventType.i, *EventProcedure)
	; +-----------------------------------------------------------------
	; |Description  : Ordnet einem Gadget eine Eventprozedur zu
	; |Arguments    : iWindowNr      : Nummer des Fensters
	; |             : iGadgetNr      : Nummer des Gadgets
	; |             : iEventType     : Eventtyp-Konstante auf die reagiert werden soll
	; |             : *EventProcedure: Zeiger auf die Prozedur, die bei diesem Eventtyp aufgerufen werden soll
	; |Results      : #False, wenn evh_Init() zuvor nicht aufgerufen wurde, sonst #True
	; +-----------------------------------------------------------------
	Protected iCount.i

	evh_Init()

	; Wenn der Event schon registriert ist, dann die neue Eventprozedur zuordnen...
	For iCount = 0 To evh_stSettings\iCurrentEntry - 1
		If evh_stEventArray(iCount)\iControlNr = iGadgetNr And evh_stEventArray(iCount)\iEventType = iEventType
			evh_stEventArray(iCount)\iEventProcedure = *EventProcedure
			ProcedureReturn #True
		EndIf
	Next iCount

	; ... ansonsten den Event registrieren
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iWindowNr = iWindowNr
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iControlType = #EVH_Controltype_Gadget
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iControlNr = iGadgetNr
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iEventType = iEventType
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iEventProcedure = *EventProcedure
	evh_stSettings\iCurrentEntry + 1
	ReDim evh_stEventArray.evh_Eventstruct(evh_stSettings\iCurrentEntry)

	ProcedureReturn #True
EndProcedure
Procedure.i evh_SetMenuEvent  (iWindowNr.i, iMenuitemNr.i, *EventProcedure)
	; +-----------------------------------------------------------------
	; |Description  : Ordnet einem Menitem eine Eventprozedur zu
	; |Arguments    : iWindowNr      : Nummer des Fensters zu dem das Men gehrt
	; |             : iMenuitemNr    : Nummer des Menitems
	; |             : *EventProcedure: Zeiger auf die Prozedur die bei diesem Eventtyp aufgerufen werden soll
	; |Results      : #False, wenn evh_Init() zuvor nicht aufgerufen wurde, sonst #True
	; +-----------------------------------------------------------------
	Protected iCount.i

	evh_Init()

	; Wenn der Event schon registriert ist, dann die neue Eventprozedur zuordnen...
	For iCount = 0 To evh_stSettings\iCurrentEntry - 1
		If evh_stEventArray(iCount)\iControlNr = iMenuitemNr And evh_stEventArray(iCount)\iWindowNr = iWindowNr
			evh_stEventArray(iCount)\iEventProcedure = *EventProcedure
			ProcedureReturn #True
		EndIf
	Next iCount

	; ... ansonsten den Event registrieren
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iWindowNr = iWindowNr
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iControlType = #EVH_Controltype_Menu
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iControlNr = iMenuitemNr
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iEventType = 0
	evh_stEventArray(evh_stSettings\iCurrentEntry)\iEventProcedure = *EventProcedure
	evh_stSettings\iCurrentEntry + 1
	ReDim evh_stEventArray.evh_Eventstruct(evh_stSettings\iCurrentEntry)

	ProcedureReturn #True
EndProcedure
;-
Procedure.i evh_RemoveEvent(iWindowNr.i, iItemNr.i = -1, iEventType.i = -1)
	; +-----------------------------------------------------------------
	; |Description  : Entfernt den registrierten Eventeintrag eines Gadgets
	; |Arguments    : iWindowNr      : Nummer des Fensters zu dem der Event gehrt
	; |             :                : Sind die folgenden zwei Parameter = 0, dann werden alle
	; |             :                : Events entfernt, die zu diesem Fenster registriert wurden.
	; |             : iItemNr        : Nummer des Gadgets, Menuitems oder Windows
	; |             : iEventType     : Eventtyp-Konstante die zuvor registriert wurde (Bei MenuEvents = 0)
	; |Results      : #False, wenn evh_Init() zuvor nicht aufgerufen wurde oder der 
	; |             : Event zuvor gar nicht registriert wurde, sonst #True
	; +-----------------------------------------------------------------
	Protected iCount.i, iEventsRemoved.i, iAllEventsFromWindow.i = #False
	Protected iExitcode.i = #False

	evh_Init()
	
	; Wenn nur iWindowNr bergeben wird, dann alle Events dieses Fensters lschen
	If iItemNr = -1 And iEventType = -1
		iAllEventsFromWindow = #True
	EndIf
	
	; Registrierte(n) Event(s) in der Liste suchen
	For iCount = 0 To evh_stSettings\iCurrentEntry - 1
		If (iAllEventsFromWindow = #False And evh_stEventArray(iCount)\iWindowNr = iWindowNr And evh_stEventArray(iCount)\iControlNr = iItemNr And evh_stEventArray(iCount)\iEventType = iEventType) Or (iAllEventsFromWindow = #True And evh_stEventArray(iCount)\iWindowNr = iWindowNr)
			; Eintrag mit dem vorletzten Eintrag des Array tauschen (im letzten stehen nur Nullen)
			evh_stEventArray(iCount)\iWindowNr       = evh_stEventArray(evh_stSettings\iCurrentEntry - 1)\iWindowNr
			evh_stEventArray(iCount)\iControlNr      = evh_stEventArray(evh_stSettings\iCurrentEntry - 1)\iControlNr
			evh_stEventArray(iCount)\iControlType    = evh_stEventArray(evh_stSettings\iCurrentEntry - 1)\iControlType
			evh_stEventArray(iCount)\iEventType      = evh_stEventArray(evh_stSettings\iCurrentEntry - 1)\iEventType
			evh_stEventArray(iCount)\iEventProcedure = evh_stEventArray(evh_stSettings\iCurrentEntry - 1)\iEventProcedure
			iEventsRemoved + 1
			iExitcode = #True
		EndIf
	Next iCount
	
	; und das Array krzen
			evh_stSettings\iCurrentEntry - iEventsRemoved
	ReDim evh_stEventArray.evh_Eventstruct(evh_stSettings\iCurrentEntry)
	
	ProcedureReturn iExitcode
EndProcedure
;-
Procedure.i evh_ProcessEvents(iTimeOut.i, iObserveWindow.i, *OwnEventHandler = 0, *stEvents.evh_EventArgsStruct = 0)
	; +-----------------------------------------------------------------
	; |Description  : Ruft die Event-Prozedur des entsprechenden Gadgets auf, falls es zuvor registriert wurde
	; |             : Ruft danach optional eine eigene Eventroutine auf
	; |Arguments    : iTimeOut        : Parameter fr WaitWindowEvent
	; |             : iObserveWindow  : Nummer des Fensters, dessen #PB_Event_CloseWindow Event berwacht werden soll
	; |             :   Tritt #PB_Event_CloseWindow fr dieses Fenster auf, gibt die Prozedur #False zurck
	; |             : *OwnEventHandler: Zeiger auf eine optionale Event-Behandlungsprozedur
	; |             :   Die Eventprozedur bekommt als Parameter einen Pointer auf eine evh_EventArgsStruct bergeben:
	; |             : *stEvents       : Zeiger auf eine optionale Eventstruktur, mit der zu bearbeitende Events bergeben
	; |             :   werden knnen.
	; |             :   Ist stEvents initialisiert (!= 0), dann ermittelt evh_ProcessEvents() die Events nicht selbst,
	; |             :   sondern greift auf die Events der Struktur zu. Ist der Strukturzeiger = 0, dann werden die
	; |             :   Purebasic-Events selbst ermittelt.
	; |Results      : #False, wenn #PB_Event_CloseWindow fr das berwachte Fenster aufgetreten ist, andernfalls #True
	; +-----------------------------------------------------------------
	Protected iCount.i
	Protected evh_stEventArgs.evh_EventArgsStruct
	Protected evh_CallEventProcedure.evh_CallEventProcedure
	Protected evh_CallOwnEventHandler.evh_CallOwnEventHandler

	If *stEvents <> 0 
	  evh_stEventArgs\iEvent       = *stEvents\iEvent
	  evh_stEventArgs\iEventType   = *stEvents\iEventType
	  evh_stEventArgs\iEventWindow = *stEvents\iEventWindow
	  evh_stEventArgs\iEventGadget = *stEvents\iEventGadget
	  evh_stEventArgs\iEventMenu   = *stEvents\iEventMenu
	  evh_stEventArgs\iEventTimer  = *stEvents\iEventTimer
	  evh_stEventArgs\iEventwParam = *stEvents\iEventwParam
	  evh_stEventArgs\iEventlParam = *stEvents\iEventlParam
	Else
	  evh_stEventArgs\iEvent       = WaitWindowEvent(iTimeOut)
	  evh_stEventArgs\iEventType   = EventType()
	  evh_stEventArgs\iEventWindow = EventWindow()
	  evh_stEventArgs\iEventGadget = EventGadget()
	  evh_stEventArgs\iEventMenu   = EventMenu()
	  evh_stEventArgs\iEventTimer  = EventTimer()
	  CompilerIf #PB_Compiler_OS = #PB_OS_Windows
	  	evh_stEventArgs\iEventwParam = EventwParam()
	  	evh_stEventArgs\iEventlParam = EventlParam()
	  CompilerElse
	  	evh_stEventArgs\iEventwParam = 0
	  	evh_stEventArgs\iEventlParam = 0
	  CompilerEndIf
	EndIf

	; Prfen, ob ein Event fr ein zuvor registriertes Gadget/Menitem aufgetreten ist
	If evh_stSettings\iCurrentEntry > 0 And evh_stSettings\iHandlerInitialized And evh_stEventArgs\iEvent > 0
		For iCount = 0 To evh_stSettings\iCurrentEntry - 1

			; Windowevents prfen
			If evh_stEventArray(iCount)\iControlType = #EVH_Controltype_Window
				If evh_stEventArray(iCount)\iControlNr = evh_stEventArgs\iEventWindow And evh_stEventArray(iCount)\iEventType = evh_stEventArgs\iEvent
					evh_CallEventProcedure = evh_stEventArray(iCount)\iEventProcedure
					evh_CallEventProcedure(@evh_stEventArgs)
				EndIf
			EndIf

			; Gadgetevents prfen
			If evh_stEventArgs\iEvent = #PB_Event_Gadget And evh_stEventArray(iCount)\iControlType = #EVH_Controltype_Gadget
				If evh_stEventArray(iCount)\iWindowNr = evh_stEventArgs\iEventWindow And evh_stEventArray(iCount)\iControlNr = evh_stEventArgs\iEventGadget And evh_stEventArray(iCount)\iEventType = evh_stEventArgs\iEventType
					evh_CallEventProcedure = evh_stEventArray(iCount)\iEventProcedure
					evh_CallEventProcedure(@evh_stEventArgs)
				EndIf
			EndIf

			; Menevents prfen
			If evh_stEventArgs\iEvent = #PB_Event_Menu And evh_stEventArray(iCount)\iControlType = #EVH_Controltype_Menu
				If evh_stEventArray(iCount)\iWindowNr = evh_stEventArgs\iEventWindow And evh_stEventArray(iCount)\iControlNr = evh_stEventArgs\iEventMenu
					evh_CallEventProcedure = evh_stEventArray(iCount)\iEventProcedure
					evh_CallEventProcedure(@evh_stEventArgs)
				EndIf
			EndIf

		Next iCount
	EndIf

	; Die optionale, eigene Eventroutine aufrufen
	If *OwnEventHandler > 0
		evh_CallOwnEventHandler = *OwnEventHandler
		evh_CallOwnEventHandler(@evh_stEventArgs)
	EndIf
	
	; Prfen, ob das zu berwachende Fenster geschlossen wurde
	If evh_stEventArgs\iEvent = #PB_Event_CloseWindow And evh_stEventArgs\iEventWindow = iObserveWindow 
		ProcedureReturn #False
	Else
		ProcedureReturn #True
	EndIf

EndProcedure
